@@ -17,11 +17,13 @@
 handler_ascii_alert <- function(symbol = "\a", file = stderr(), intrusiveness = getOption("progressr.intrusiveness.auditory", 5.0), target = c("terminal", "audio"), ...) {
   reporter <- local({
     list(
+      hide   = function(...) NULL,
+      unhide = function(...) NULL,
       update = function(config, state, progression, ...) {
         if (state$enabled && progression$amount != 0) cat(file = file, symbol)
       }
     )
   })
 
-  make_progression_handler("ascii_alert", reporter, intrusiveness = intrusiveness, ...)
+  make_progression_handler("ascii_alert", reporter, intrusiveness = intrusiveness, target = target, ...)
 }
